==> flutter_mylibrary-0.2.25.tar.gz: lib/base/provider/view_state_refresh_list_model.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_mylibrary/base/provider/request_info.dart';
import 'package:flutter_mylibrary/base/provider/view_state_model.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/dio_utils.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';


/// 基于
abstract class ViewStateRefreshListModel<T> extends ViewStateModel {

  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  RequestInfo getRequestInfo();

  /// 页面数据
  List<T> list = [];

  onCompleted(List<T> data) {}


  ///获取数据
  Future<List<T>> getData() {
    RequestInfo requestInfo = getRequestInfo();
    return DioUtils.getInstance().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onFinish: (){
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
        },
        onError: isInit? null : (e){
          setError(e, e.message, message: e.message);
        }
    ).then((res) => res.data is List? List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data));
  }


  void initData(){
    refresh(init: true);
  }



  ///获取一个公共的刷新控件
  Widget getEasyRefreshWidget({ViewStateModelWidgetBuilder<List<T>> builder, Widget emptyWidget, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true}){
    Widget emptyErrorWidget = getRefreshNoDataWidget(viewStateModel: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget);
    return EasyRefresh(
      emptyWidget: emptyErrorWidget,
      child: isNotEmpty(emptyErrorWidget)? builder(list) : Container(),
      onRefresh: isBusy? null : () async{
        refresh();
      },
      onLoad: (isBusy || !hasMore || !getRequestInfo().useLoadMore)? null : () async{
        loadMore();
      },
      controller: refreshController,
    );
  }

  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  /// 下拉刷新
  ///
  /// [init] 是否是第一次加载
  /// true:  Error时,需要跳转页面
  /// false: Error时,不需要跳转页面,直接给出提示
  Future<List<T>> refresh({bool init = false}) async {
    hasMore = true;
    try {
      _currentPageNum = firstPage();
      var data = await getData();
      if (data.isEmpty) {
        hasMore = false;
        refreshController.resetRefreshState();
        list.clear();
        setEmpty();
      } else {
        list.clear();
        list.addAll(data);
        // 小于分页的数量,禁止上拉加载更多
        if (data.length < pageSize()) {
          hasMore = false;
          refreshController.finishRefresh();
          refreshController.finishLoad();
          refreshController.resetRefreshState();
        } else {
          //防止上次上拉加载更多失败,需要重置状态
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
          refreshController.resetLoadState();
        }
        onCompleted(data);
        setIdle();
      }
      return data;
    } catch (e, s) {
      /// 页面已经加载了数据,如果刷新报错,不应该直接跳转错误页面
      /// 而是显示之前的页面数据.给出错误提示
      if (init){
        list.clear();
        setError(e, s);
      }
      hasMore = false;
      refreshController.finishRefresh();
      refreshController.finishLoad();
      return list;
    }
  }

  /// 上拉加载更多
  Future<List<T>> loadMore() async {
    try {
      if (hasMore) {
        _currentPageNum = ++_currentPageNum;
        var data = await getData();
        if (data.isEmpty) {
          _currentPageNum--;
          hasMore = false;
          refreshController.finishLoad();
        } else {
          list.addAll(data);
          if (data.length < pageSize()) {
            hasMore = false;
            refreshController.finishLoad();
          } else {
            hasMore = true;
            refreshController.finishLoad();
          }
          notifyListeners();
        }
        onCompleted(data);
        return data;
      }else{
        ///没有更多了
        refreshController.finishLoad(noMore: true);
        return list;
      }
    } catch (e, s) {
      _currentPageNum--;
      refreshController.finishLoad(success: false, noMore: true);
      debugPrint('error--->\n' + e.toString());
      debugPrint('statck--->\n' + s.toString());
      return list;
    }
  }

  bool _hasMore;

  ///设置是否还有更多
  set hasMore(bool hasMore){
    this._hasMore = hasMore;
  }

  bool get hasMore => _hasMore??true;

  @override
  void dispose() {
    refreshController.dispose();
    super.dispose();
  }


}

==> flutter_mylibrary-0.2.27.tar.gz: lib/base/provider/view_state_refresh_list_model.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_mylibrary/base/provider/request_info.dart';
import 'package:flutter_mylibrary/base/provider/view_state_model.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/dio_utils.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';


/// 基于
abstract class ViewStateRefreshListModel<T> extends ViewStateModel {

  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  RequestInfo getRequestInfo();

  /// 页面数据
  List<T> list = [];

  onCompleted(List<T> data) {}


  ///获取数据
  Future<List<T>> getData() {
    RequestInfo requestInfo = getRequestInfo();
    return DioUtils.getInstance().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onFinish: (){
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
        },
        onError: isInit? null : (e){
          setError(e, e.message, message: e.message);
        }
    ).then((res) => res.data is List? List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data));
  }


  void initData(){
    refresh(init: true);
  }



  ///获取一个公共的刷新控件
  Widget getEasyRefreshWidget({ViewStateModelWidgetBuilder<List<T>> builder, Widget emptyWidget, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true}){
    Widget emptyErrorWidget = getRefreshNoDataWidget(viewStateModel: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget);
    return EasyRefresh(
      emptyWidget: emptyErrorWidget,
      child: isEmpty(emptyErrorWidget)? builder(list) : Container(),
      onRefresh: isBusy? null : () async{
        refresh();
      },
      onLoad: (isBusy || !hasMore || !getRequestInfo().useLoadMore)? null : () async{
        loadMore();
      },
      controller: refreshController,
    );
  }

  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  /// 下拉刷新
  ///
  /// [init] 是否是第一次加载
  /// true:  Error时,需要跳转页面
  /// false: Error时,不需要跳转页面,直接给出提示
  Future<List<T>> refresh({bool init = false}) async {
    hasMore = true;
    try {
      _currentPageNum = firstPage();
      var data = await getData();
      if (data.isEmpty) {
        hasMore = false;
        refreshController.resetRefreshState();
        list.clear();
        setEmpty();
      } else {
        list.clear();
        list.addAll(data);
        // 小于分页的数量,禁止上拉加载更多
        if (data.length < pageSize()) {
          hasMore = false;
          refreshController.finishRefresh();
          refreshController.finishLoad();
          refreshController.resetRefreshState();
        } else {
          //防止上次上拉加载更多失败,需要重置状态
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
          refreshController.resetLoadState();
        }
        onCompleted(data);
        setIdle();
      }
      return data;
    } catch (e, s) {
      /// 页面已经加载了数据,如果刷新报错,不应该直接跳转错误页面
      /// 而是显示之前的页面数据.给出错误提示
      if (init){
        list.clear();
        setError(e, s);
      }
      hasMore = false;
      refreshController.finishRefresh();
      refreshController.finishLoad();
      return list;
    }
  }

  /// 上拉加载更多
  Future<List<T>> loadMore() async {
    try {
      if (hasMore) {
        _currentPageNum = ++_currentPageNum;
        var data = await getData();
        if (data.isEmpty) {
          _currentPageNum--;
          hasMore = false;
          refreshController.finishLoad();
        } else {
          list.addAll(data);
          if (data.length < pageSize()) {
            hasMore = false;
            refreshController.finishLoad();
          } else {
            hasMore = true;
            refreshController.finishLoad();
          }
          notifyListeners();
        }
        onCompleted(data);
        return data;
      }else{
        ///没有更多了
        refreshController.finishLoad(noMore: true);
        return list;
      }
    } catch (e, s) {
      _currentPageNum--;
      refreshController.finishLoad(success: false, noMore: true);
      debugPrint('error--->\n' + e.toString());
      debugPrint('statck--->\n' + s.toString());
      return list;
    }
  }

  bool _hasMore;

  ///设置是否还有更多
  set hasMore(bool hasMore){
    this._hasMore = hasMore;
  }

  bool get hasMore => _hasMore??true;

  @override
  void dispose() {
    refreshController.dispose();
    super.dispose();
  }


}
